@@ -16,8 +16,11 @@
 ***************************************************************************/
 
 #include "httpbrokerclient.h"
+#include <QNetworkAccessManager>
 #include <QUrl>
-#include <QHttp>
+#include <QNetworkRequest>
+#include <QNetworkReply>
+#include <QUuid>
 #include <QTextStream>
 #include <QFile>
 #include <QDir>
@@ -34,6 +37,7 @@
 #include <QTemporaryFile>
 #include <QInputDialog>
 
+
 HttpBrokerClient::HttpBrokerClient ( ONMainWindow* wnd, ConfigFile* cfg )
 {
     config=cfg;
@@ -42,6 +46,7 @@ HttpBrokerClient::HttpBrokerClient ( ONMainWindow* wnd, ConfigFile* cfg )
     QUrl lurl ( config->brokerurl );
     if(lurl.userName().length()>0)
         config->brokerUser=lurl.userName();
+    nextAuthId=config->brokerUserId;
 
     if(config->brokerurl.indexOf("ssh://")==0)
     {
@@ -56,30 +61,19 @@ HttpBrokerClient::HttpBrokerClient ( ONMainWindow* wnd, ConfigFile* cfg )
     {
         sshBroker=false;
 
-        http=new QHttp ( this );
-
-        if ( config->brokerurl.indexOf ( "https://" ) ==0 ) {
-            if ((config->brokerCaCertFile.length() >0) && (QFile::exists(config->brokerCaCertFile))) {
-
-                sslSocket = new QSslSocket(this);
-                connect ( sslSocket, SIGNAL ( sslErrors ( const QList<QSslError>& ) ),this,
-                          SLOT ( slotSslErrors ( const QList<QSslError>& ) ) );
-                http->setSocket(sslSocket);
-                sslSocket->addCaCertificates(config->brokerCaCertFile, QSsl::Pem);
-                x2goDebug<<"Custom CA certificate file loaded into HTTPS broker client: "<<config->brokerCaCertFile;
-
-            } else {
-                connect ( http, SIGNAL ( sslErrors ( const QList<QSslError>& ) ),this,
-                          SLOT ( slotSslErrors ( const QList<QSslError>& ) ) );
-            }
-            http->setHost ( lurl.host(),QHttp::ConnectionModeHttps,
-                            lurl.port ( 443 ) );
-        } else {
-            http->setHost ( lurl.host(),QHttp::ConnectionModeHttp,
-                            lurl.port ( 80 ) );
+        if ((config->brokerCaCertFile.length() >0) && (QFile::exists(config->brokerCaCertFile))) {
+            QSslSocket::addDefaultCaCertificates(config->brokerCaCertFile, QSsl::Pem);
+            x2goDebug<<"Custom CA certificate file loaded into HTTPS broker client: "<<config->brokerCaCertFile;
         }
-        connect ( http,SIGNAL ( requestFinished ( int,bool ) ),this,
-                  SLOT ( slotRequestFinished ( int,bool ) ) );
+
+        http=new QNetworkAccessManager ( this );
+        x2goDebug<<"Setting up connection to broker: "<<config->brokerurl;
+
+        connect ( http, SIGNAL ( sslErrors ( QNetworkReply*, const QList<QSslError>& ) ),this,
+                  SLOT ( slotSslErrors ( QNetworkReply*, const QList<QSslError>& ) ) );
+
+        connect ( http,SIGNAL ( finished (QNetworkReply*) ),this,
+                  SLOT ( slotRequestFinished (QNetworkReply*) ) );
     }
 }
 
@@ -93,13 +87,13 @@ void HttpBrokerClient::createSshConnection()
     QUrl lurl ( config->brokerurl );
     sshConnection=new SshMasterConnection (this, lurl.host(), lurl.port(22),false,
                                            config->brokerUser, config->brokerPass,config->brokerSshKey,config->brokerAutologin,
-					   config->brokerKrbLogin, false);
+                                           config->brokerKrbLogin, false);
 
     connect ( sshConnection, SIGNAL ( connectionOk(QString)), this, SLOT ( slotSshConnectionOk() ) );
     connect ( sshConnection, SIGNAL ( serverAuthError ( int,QString, SshMasterConnection* ) ),this,
               SLOT ( slotSshServerAuthError ( int,QString, SshMasterConnection* ) ) );
-    connect ( sshConnection, SIGNAL ( needPassPhrase(SshMasterConnection*)),this,
-              SLOT ( slotSshServerAuthPassphrase(SshMasterConnection*)) );
+    connect ( sshConnection, SIGNAL ( needPassPhrase(SshMasterConnection*, bool)),this,
+              SLOT ( slotSshServerAuthPassphrase(SshMasterConnection*, bool)) );
     connect ( sshConnection, SIGNAL ( userAuthError ( QString ) ),this,SLOT ( slotSshUserAuthError ( QString ) ) );
     connect ( sshConnection, SIGNAL ( connectionError(QString,QString)), this,
               SLOT ( slotSshConnectionError ( QString,QString ) ) );
@@ -199,11 +193,23 @@ void HttpBrokerClient::slotSshServerAuthError(int error, QString sshMessage, Ssh
 
 }
 
-void HttpBrokerClient::slotSshServerAuthPassphrase(SshMasterConnection* connection)
+void HttpBrokerClient::slotSshServerAuthPassphrase(SshMasterConnection* connection, bool verificationCode)
 {
     bool ok;
+    QString message;
+
+    if(verificationCode)
+    {
+        message=tr("Verification code:");
+    }
+    else
+    {
+        message=tr("Enter passphrase to decrypt a key");
+    }
+
+
     QString phrase=QInputDialog::getText(0,connection->getUser()+"@"+connection->getHost()+":"+QString::number(connection->getPort()),
-                                         tr("Enter passphrase to decrypt a key"),QLineEdit::Password,QString::null, &ok);
+                                         message, QLineEdit::Password,QString::null, &ok);
     if(!ok)
     {
         phrase=QString::null;
@@ -236,6 +242,7 @@ void HttpBrokerClient::slotSshUserAuthError(QString error)
 void HttpBrokerClient::getUserSessions()
 {
     QString brokerUser=config->brokerUser;
+    x2goDebug<<"called getUserSessions: brokeruser: "<<brokerUser<<" authid: "<<nextAuthId;
     if(mainWindow->getUsePGPCard())
         brokerUser=mainWindow->getCardLogin();
     config->sessiondata=QString::null;
@@ -246,11 +253,12 @@ void HttpBrokerClient::getUserSessions()
                              "task=listsessions&"<<
                              "user="<<brokerUser<<"&"<<
                              "password="<<config->brokerPass<<"&"<<
-                             "authid="<<config->brokerUserId;
-        QUrl lurl ( config->brokerurl );
-        httpSessionAnswer.close();
-        httpSessionAnswer.setData ( 0,0 );
-        sessionsRequest=http->post ( lurl.path(),req.toUtf8(),&httpSessionAnswer );
+                             "authid="<<nextAuthId;
+
+        x2goDebug << "sending request: "<< req.toUtf8();
+        QNetworkRequest request(QUrl(config->brokerurl));
+        request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
+        sessionsRequest=http->post (request, req.toUtf8() );
     }
     else
     {
@@ -259,8 +267,8 @@ void HttpBrokerClient::getUserSessions()
             createSshConnection();
             return;
         }
-        if (config->brokerUserId.length() > 0) {
-            sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --authid "+config->brokerUserId+ " --task listsessions",
+        if (nextAuthId.length() > 0) {
+            sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --authid "+nextAuthId+ " --task listsessions",
                                             this, SLOT ( slotListSessions ( bool, QString,int ) ));
         } else {
             sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --task listsessions",
@@ -271,6 +279,7 @@ void HttpBrokerClient::getUserSessions()
 
 void HttpBrokerClient::selectUserSession(const QString& session)
 {
+    x2goDebug<<"called selectUserSessions";
     QString brokerUser=config->brokerUser;
     if(mainWindow->getUsePGPCard())
         brokerUser=mainWindow->getCardLogin();
@@ -283,16 +292,17 @@ void HttpBrokerClient::selectUserSession(const QString& session)
                              "sid="<<session<<"&"<<
                              "user="<<brokerUser<<"&"<<
                              "password="<<config->brokerPass<<"&"<<
-                             "authid="<<config->brokerUserId;
-        QUrl lurl ( config->brokerurl );
-        httpSessionAnswer.close();
-        httpSessionAnswer.setData ( 0,0 );
-        selSessRequest=http->post ( lurl.path(),req.toUtf8(),&httpSessionAnswer );
+                             "authid="<<nextAuthId;
+        x2goDebug << "sending request: "<< req.toUtf8();
+        QNetworkRequest request(QUrl(config->brokerurl));
+        request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
+        selSessRequest=http->post (request, req.toUtf8() );
+
     }
     else
     {
-        if (config->brokerUserId.length() > 0) {
-            sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --authid "+config->brokerUserId+ " --task selectsession --sid \\\""+session+"\\\"",
+        if (nextAuthId.length() > 0) {
+            sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --authid "+nextAuthId+ " --task selectsession --sid \\\""+session+"\\\"",
                                             this,SLOT ( slotSelectSession(bool,QString,int)));
         } else {
             sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --task selectsession --sid \\\""+session+"\\\"",
@@ -317,16 +327,16 @@ void HttpBrokerClient::changePassword(QString newPass)
                              "newpass="<<newPass<<"&"<<
                              "user="<<brokerUser<<"&"<<
                              "password="<<config->brokerPass<<"&"<<
-                             "authid="<<config->brokerUserId;
-        QUrl lurl ( config->brokerurl );
-        httpSessionAnswer.close();
-        httpSessionAnswer.setData ( 0,0 );
-        chPassRequest=http->post ( lurl.path(),req.toUtf8(),&httpSessionAnswer );
+                             "authid="<<nextAuthId;
+        x2goDebug << "sending request: "<< req.toUtf8();
+        QNetworkRequest request(QUrl(config->brokerurl));
+        request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
+        chPassRequest=http->post (request, req.toUtf8() );
     }
     else
     {
-        if (config->brokerUserId.length() > 0) {
-            sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --authid "+config->brokerUserId+ " --task setpass --newpass "+newPass, this,
+        if (nextAuthId.length() > 0) {
+            sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --authid "+nextAuthId+ " --task setpass --newpass "+newPass, this,
                                             SLOT ( slotPassChanged(bool,QString,int)));
         } else {
             sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --task setpass --newpass "+newPass, this,
@@ -337,21 +347,21 @@ void HttpBrokerClient::changePassword(QString newPass)
 
 void HttpBrokerClient::testConnection()
 {
+    x2goDebug<<"called testConnection";
     if(!sshBroker)
     {
         QString req;
         QTextStream ( &req ) <<
                              "task=testcon";
-        QUrl lurl ( config->brokerurl );
-        httpSessionAnswer.close();
-        httpSessionAnswer.setData ( 0,0 );
-        requestTime.start();
-        testConRequest=http->post ( lurl.path(),req.toUtf8(),&httpSessionAnswer );
+        x2goDebug << "sending request: "<< req.toUtf8();
+        QNetworkRequest request(QUrl(config->brokerurl));
+        request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
+        testConRequest=http->post (request, req.toUtf8() );
     }
     else
     {
-        if (config->brokerUserId.length() > 0) {
-            sshConnection->executeCommand(config->sshBrokerBin+" --authid "+config->brokerUserId+ " --task testcon",
+        if (nextAuthId.length() > 0) {
+            sshConnection->executeCommand(config->sshBrokerBin+" --authid "+nextAuthId+ " --task testcon",
                                           this, SLOT ( slotSelectSession(bool,QString,int)));
         } else {
             sshConnection->executeCommand(config->sshBrokerBin+" --task testcon",
@@ -380,6 +390,7 @@ void HttpBrokerClient::createIniFile(const QString& raw_content)
 
 bool HttpBrokerClient::checkAccess(QString answer )
 {
+    x2goDebug<<"called checkAccess - answer was: "<<answer;
     if (answer.indexOf("Access granted")==-1)
     {
         QMessageBox::critical (
@@ -390,12 +401,18 @@ bool HttpBrokerClient::checkAccess(QString answer )
         return false;
     }
     config->brokerAuthenticated=true;
+    int authBegin=answer.indexOf("AUTHID:");
+    if (authBegin!=-1)
+    {
+        nextAuthId=answer.mid(authBegin+7, answer.indexOf("\n",authBegin)-authBegin-7);
+    }
     return true;
 }
 
 
 void HttpBrokerClient::slotConnectionTest(bool success, QString answer, int)
 {
+    x2goDebug<<"called slotConnectionTest";
     if(!success)
     {
         x2goDebug<<answer;
@@ -407,8 +424,8 @@ void HttpBrokerClient::slotConnectionTest(bool success, QString answer, int)
         return;
     if(!sshBroker)
     {
-        x2goDebug<<"elapsed: "<<requestTime.elapsed()<<"received:"<<httpSessionAnswer.size()<<endl;
-        emit connectionTime(requestTime.elapsed(),httpSessionAnswer.size());
+        x2goDebug<<"elapsed: "<<requestTime.elapsed()<<"received:"<<answer.size()<<endl;
+        emit connectionTime(requestTime.elapsed(),answer.size());
     }
     return;
 
@@ -459,36 +476,38 @@ void HttpBrokerClient::slotSelectSession(bool success, QString answer, int)
 }
 
 
-void HttpBrokerClient::slotRequestFinished ( int id, bool error )
+void HttpBrokerClient::slotRequestFinished ( QNetworkReply*  reply )
 {
-//   	x2goDebug<<"http request "<<id<<", finished with: "<<error;
-
-    if ( error )
+    if(reply->error() != QNetworkReply::NoError)
     {
-        x2goDebug<<http->errorString();
-        QMessageBox::critical(0,tr("Error"),http->errorString());
+        x2goDebug<<"Broker HTTP request failed with error: "<<reply->errorString();
+        QMessageBox::critical(0,tr("Error"),reply->errorString());
         emit fatalHttpError();
         return;
     }
 
-    QString answer ( httpSessionAnswer.data() );
-    x2goDebug<<"cmd request answer: "<<answer;
-    if (id==testConRequest)
+    QString answer ( reply->readAll() );
+    x2goDebug<<"A http request returned.  Result was: "<<answer;
+    if (reply == testConRequest)
     {
         slotConnectionTest(true,answer,0);
     }
-    if (id == sessionsRequest)
+    if (reply == sessionsRequest)
     {
         slotListSessions(true, answer,0);
     }
-    if (id == selSessRequest)
+    if (reply == selSessRequest)
     {
         slotSelectSession(true,answer,0);
     }
-    if ( id == chPassRequest)
+    if (reply == chPassRequest)
     {
         slotPassChanged(true,answer,0);
     }
+
+    // We receive ownership of the reply object
+    // and therefore need to handle deletion.
+    reply->deleteLater();
 }
 
 void HttpBrokerClient::parseSession(QString sinfo)
@@ -525,7 +544,7 @@ void HttpBrokerClient::parseSession(QString sinfo)
 }
 
 
-void HttpBrokerClient::slotSslErrors ( const QList<QSslError> & errors )
+void HttpBrokerClient::slotSslErrors ( QNetworkReply* netReply, const QList<QSslError> & errors )
 {
     QStringList err;
     QSslCertificate cert;
@@ -552,7 +571,7 @@ void HttpBrokerClient::slotSslErrors ( const QList<QSslError> & errors )
         QSslCertificate mcert ( &fl );
         if ( mcert==cert )
         {
-            http->ignoreSslErrors();
+            netReply->ignoreSslErrors();
             requestTime.restart();
             return;
         }
@@ -623,7 +642,7 @@ void HttpBrokerClient::slotSslErrors ( const QList<QSslError> & errors )
         fl.open ( QIODevice::WriteOnly | QIODevice::Text );
         QTextStream ( &fl ) <<cert.toPem();
         fl.close();
-        http->ignoreSslErrors();
+        netReply->ignoreSslErrors();
         x2goDebug<<"store certificate in  "<<homeDir+"/.x2go/ssl/exceptions/"+
                  lurl.host() +"/"+fname;
         requestTime.restart();
