@@ -47,6 +47,7 @@
 
 #ifdef Q_OS_WIN
 #include <windows.h>
+#include <QSysInfo>
 #endif
 /**
 @author Oleksandr Shneyder
@@ -492,6 +493,10 @@ public:
     {
         return sessionCfg;
     }
+    bool getHideFolderSharing()
+    {
+        return hideFolderSharing;
+    }
 
     void runApplication(QString exec);
 
@@ -546,7 +551,10 @@ private:
     bool drawMenu;
     bool extStarted;
     bool startMaximized;
+    bool closeDisconnect;
     bool startHidden;
+    bool keepTrayIcon;
+    bool hideFolderSharing;
     bool defaultUseSound;
     bool defaultXinerama;
     bool cardStarted;
@@ -772,6 +780,7 @@ private:
     PROCESS_INFORMATION sshd;
     bool winSshdStarted;
     static QString u3Device;
+    bool pulseStarted;
 
     QString pulseVersionTestOutput;
     QProcess* pulseVersionTest;
@@ -1103,7 +1112,10 @@ private:
     void initSelectSessDlg();
     void initStatusDlg();
     void initPassDlg();
-    void printSshDError();
+    void printSshDError_startupFailure();
+    void printSshDError_noHostPubKey();
+    void printSshDError_noExportPubKey();
+    void printSshDError_noAuthorizedKeysFile();
     void loadPulseModuleNativeProtocol();
     void initEmbedToolBar();
 #ifdef Q_OS_LINUX
