@@ -31,11 +31,7 @@
 #undef DEBUG
 // #define DEBUG
 
-#ifdef Q_OS_DARWIN
 #define KEEPALIVE_OPTION " -o ServerAliveInterval=60 "
-#else
-#define KEEPALIVE_OPTION " -o ProtocolKeepAlives=60 "
-#endif
 
 SshProcess::SshProcess(SshMasterConnection* master, int pid): QObject(0)
 {
@@ -221,13 +217,10 @@ void SshProcess::startNormal(const QString& cmd)
 #endif
                           QString::number(masterCon->getPort())+" -l "+
                           masterCon->getUser()+" "+ host +  " \""+shcmd+"\"";
-#ifdef DEBUG
-        x2goDebug<<"running ssh:" <<sshString<<endl;
-#endif
         procUuid=uuidStr;
         proc->start(sshString);
 
-        if (!proc->waitForStarted(5000))
+        if (!proc->waitForStarted(15000))
         {
             stdErrString=proc->errorString();
 #ifdef DEBUG
@@ -274,7 +267,7 @@ void SshProcess::start_cp(QString src, QString dst)
 #endif
         proc->start(sshString);
 
-        if (!proc->waitForStarted(5000))
+        if (!proc->waitForStarted(15000))
         {
             stdErrString=proc->errorString();
 #ifdef DEBUG
@@ -304,11 +297,6 @@ void SshProcess::startTunnel(const QString& forwardHost, uint forwardPort, const
         this->localPort=localPort;
         if (!reverse)
             tunnelLoop();
-        else
-        {
-            connect(masterCon, SIGNAL(reverseListenOk(SshProcess*)), this, SLOT(slotReverseTunnelOk(SshProcess*)));
-            tunnelConnection=masterCon->reverseTunnelConnection(this, forwardPort, localHost, localPort);
-        }
     }
     else
     {
@@ -327,7 +315,7 @@ void SshProcess::startTunnel(const QString& forwardHost, uint forwardPort, const
             sshString+=" -R "+ QString::number(forwardPort)+":"+forwardHost+":"+QString::number(localPort);
 
 #ifdef DEBUG
-        x2goDebug<<"running ssh:" <<sshString<<endl;
+        x2goDebug<<"TUNNEL: running ssh:" <<sshString<<endl;
 #endif
         proc->start(sshString);
 
@@ -352,7 +340,7 @@ void SshProcess::slotStdErr(SshProcess* creator, QByteArray data)
     if (creator!=this)
         return;
 #ifdef DEBUG
-    x2goDebug<<"new err data:"<<data<<endl;
+//     x2goDebug<<"new err data:"<<data<<endl;
 #endif
     stdErrString+=data;
 
@@ -365,6 +353,9 @@ void SshProcess::slotStdErr(SshProcess* creator, QByteArray data)
 #endif
         {
             tunnelOkEmited=true;
+#ifdef DEBUG
+            x2goDebug<<"Tunnel OK"<<endl;
+#endif
             emit sshTunnelOk(pid);
         }
     }
@@ -409,6 +400,13 @@ void SshProcess::slotReverseTunnelOk(SshProcess* creator)
         emit sshTunnelOk(pid);
 }
 
+void SshProcess::slotReverseTunnelFailed(SshProcess* creator, QString error)
+{
+    if (creator==this)
+        emit sshFinished(false,error,pid);
+
+}
+
 
 void SshProcess::slotChannelClosed(SshProcess* creator, QString uuid)
 {
