@@ -481,6 +481,13 @@
         <translation>Choisir un dossier</translation>
     </message>
 </context>
+<context>
+    <name>HelpDialog</name>
+    <message>
+        <source>Options</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 <context>
     <name>HttpBrokerClient</name>
     <message>
@@ -606,6 +613,10 @@ Public key hash: </source>
         <source>Add exception</source>
         <translation>Ajouter une exception</translation>
     </message>
+    <message>
+        <source>Verification code:</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>ONMainWindow</name>
@@ -2155,10 +2166,6 @@ sound system connections through firewalls</source>
         <source>Authentication failed</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>channel_forward_listen failed</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Can not open file </source>
         <translation type="unfinished"></translation>
